<?php
/************************************************************************
 * Mailer v0.2.1-FINAL                                Start: 11/26/2009 *
 * ===================                          Last change: 11/26/2009 *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * File              : what-list_network_params.php                     *
 * -------------------------------------------------------------------- *
 * Short description : List/manage network API query parameters         *
 * -------------------------------------------------------------------- *
 * Kurzbeschreibung  : Verwaltet API-Abfrageparameter                   *
 * -------------------------------------------------------------------- *
 * $Revision::                                                        $ *
 * $Date::                                                            $ *
 * $Tag:: 0.2.1-FINAL                                                 $ *
 * $Author::                                                          $ *
 * -------------------------------------------------------------------- *
 * Copyright (c) 2003 - 2009 by Roland Haeder                           *
 * Copyright (c) 2009 - 2011 by Mailer Developer Team                   *
 * For more information visit: http://mxchange.org                      *
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,               *
 * MA  02110-1301  USA                                                  *
 ************************************************************************/

// Some security stuff...
if ((!defined('__SECURITY')) || (!isAdmin())) {
	die();
} // END - if

// Add description as navigation point
addYouAreHereLink('admin', __FILE__);

// These are expert settings!
if (doVerifyExpertSettings() != 'agreed') {
	// Abort here
	return;
} // END - if

// By default we should display list/add new forms
$GLOBALS['network_display'] = true;

// Handle form here
doNetworkHandleForm();

// Display forms?
if ($GLOBALS['network_display'] === false) {
	// Abort here
	return;
} // END - if

// Do we have a network selected?
if (isGetRequestElementSet('network')) {
	// Get its data for template
	$networkData = getNetworkDataById(getRequestElement('network'));
	$networkData['network_type_id'] = 0;

	// Init disabled list
	$GLOBALS['network_params_disabled'] = array();

	// Only show params from given type handler if choosen
	$ADD = '';
	if ((isPostRequestElementSet('network_type_id')) && (!is_array(postRequestElement('network_type_id')))) {
		// Only show from current type handler
		$ADD = sprintf(" AND p.`network_type_id`=%s", bigintval(postRequestElement('network_type_id')));

		// Overwrite type id
		$networkData['network_type_id'] = bigintval(postRequestElement('network_type_id'));

		// Display message
		displayMessage('{--ADMIN_NETWORK_REQUEST_PARAMETER_LIST_LIMITED_TO_TYPE--}');
	} // END - if

	// Load all advert types for given network
	$result = SQL_QUERY_ESC("SELECT
	p.`network_param_id`, t.`network_type_id`, t.`network_type_handle`, p.`request_param_key`, p.`request_param_value`, p.`request_param_default`
FROM
	`{?_MYSQL_PREFIX?}_network_request_params` AS p
INNER JOIN
	`{?_MYSQL_PREFIX?}_network_types` AS t
ON
	p.`network_type_id`=t.`network_type_id`
WHERE
	p.`network_id`=%s".$ADD."
ORDER BY
	t.`network_type_handle` ASC,
	p.`network_param_id` ASC",
		array(getRequestElement('network')), __FILE__, __LINE__);

	// Do we have entries?
	if (!SQL_HASZERONUMS($result)) {
		// List all entries
		$OUT = '';
		while ($row = SQL_FETCHARRAY($result)) {
			// Add it to network if the type matches selected
			if ($row['network_type_id'] == postRequestElement('network_type_id')) {
				// Matches, then add it for disabling this entry
				$GLOBALS['network_params_disabled'][$row['request_param_key']] = true;
			} // END - if

			// Is default value empty or null?
			$row['request_param_default'] = fixNullEmptyToDashes($row['request_param_default'], 3);

			// Add row template and switch color
			$OUT .= loadTemplate('admin_list_network_params_row', true, $row);
		} // END - while

		// Load main template
		loadTemplate('admin_list_network_params', false, $OUT);
	} else {
		// No entries found
		displayMessage('{%message,ADMIN_NETWORK_REQUEST_PARAMETER_404=' . $networkData['network_title'] . '%}');
	}

	// Free result
	SQL_FREERESULT($result);

	// Add form for adding new params
	loadTemplate('admin_add_network_params', false, $networkData);
} else {
	// Generate network list for this script
	outputHtml(generateAdminNetworkList());
}

// [EOF]
?>
